################################################################################
##                                                                            ##
##   IPHOD_SEARCH.PL by Kenny Vaden                                           ##
##                                                                            ##
##   Simple search function, based on specified values in SEARCH_VALS.TXT     ##
##                                                                            ##
##   Orig. Code Jan 14, 2004; Most recent update March 17, 2009               ##
##                                                                            ##
################################################################################
##                                                                            ##
##  The output file contains word-rows in identical format to the whole file  ##
##  that is being mined; within the desired range for each value, as          ##
##  specified in cvc_urquery.txt.                                             ##
##                                                                            ##
################################################################################
##  INPUT Files:  IPhOD_Reals_1.3.txt  OR  IPhOD_Pseudos_1.3_##.txt           ##
##                search_vals.txt, vowels.txt                                 ##
##                                                                            ##
##  OUTPUT File: Output.txt                                                   ##
################################################################################

data_mine();

sub data_mine {

    $query_file = "search_vals.txt";  # Tab-delimited text, search ranges

    $output_file = "Output.txt";      # Tab-delimited output, filtered results

    $number_of_matches = 0;           # This is updated to give # results

    $heder = 1; # 1 (default) = header row included in output; 0 = don't include

    $wORps = 0; # Detects if words or pseudos are in the directory (NOT BOTH!)

    load_query(); load_output(); words_or_pseudowords();


    if ($wORps == 0) {  # NEITHER WORDS NOR PSEUDOWORD FILES DETECTED (ERROR)
        print "ERROR: No word or pseudoword files detected in the current directory. No data to search.\n";
    }

    if ($wORps == 1) {  # WORD SEARCH
        print "Processing IPhOD word file.\n";
        $input_file  = "IPhOD_Reals_1.4.txt"; load_input(); search_data();
    }

    if ($wORps == 2) { # PSEUDOWORD SEARCH
        print "Processing IPhOD pseudoword files.\n";
        for ($fnum=2; $fnum<18; $fnum++) {
            ## query either does not contain a phoneme length constraint, or this file meets it.
            if ((($query[1][2] eq "") and ($query[2][2] eq "")) or ($query[1][2] <= $fnum) and ($query[2][2] >= $fnum)) {
                $input_file = "IPhOD_Pseudos_1.4_".$fnum.".txt";
                print "Processing file  ".$fnum."\n";
                load_input(); search_data(); $data = [];
            }
        }
    }

    if ($wORps == 4) {  # WORD AND PSEUDOWORD FILES DETECTED IN SAME DIR! (ERROR)
        print "ERROR: Word and Pseudoword files cannot be searched in the same directory! \nMove one group of files out.\n";
    }


    print "Finished Searching. ".$number_of_matches." matches found.\n";
    close(OUT); print "Output file closed.\n";

    end_program();
}


sub load_query {
    my ($line, @line, $row, $cell);

    open (QUE, $query_file) || die "Cannot open ".$query_file."\n";
    print $query_file." opened.\n";

    $row = 0;
    while ($line = <QUE>) {
    chomp ($line);
    $line =~ s/\"//g; # this removes any quotation marks
    if ($line ne "") {
        @line = split ("\t", $line);
            if (@line[0] ne "") {
            for ($cell=0;$cell<3;$cell++) {
            $query[$cell][$row] = @line[$cell];
            }
            }
        $row += 1;
    }
    }
    print "Query array Created, ".$row." rows.\n";
    close(QUE); print $query_file." closed.\n\n";
    $queryrows = $row;
}


sub load_output {
    open (OUT, ">".$output_file) || die "Cannot open ".$output_file."\n";
    print "Output file: ".$output_file." open.\n\n";
    print OUT $headerline."\n" if ($heder == 1); ## OUTPUT THE HEADER
}


# determine if there are words, pseudowords, both or neither in the current directory
sub words_or_pseudowords {
    my (@files, $Rfilez, $Pfilez);
    opendir (DIR, '.') or die "Can't open current dir: $!\n";
    @files = grep (!/^\.\.?$/, readdir (DIR));
    closedir (DIR); $Rfilez = 0; $Pfilez = 0;
    foreach $filenm (@files) {
        $Rfilez = 1 if $filenm =~ m/IPhOD_Reals_\d.\d.txt/;
        $Pfilez = 1 if $filenm =~ m/IPhOD_Pseudos_1.3_\d/;
    }
    $wORps = 0 if (($Rfilez == 0) and ($Pfilez == 0));
    $wORps = 1 if (($Rfilez == 1) and ($Pfilez == 0));
    $wORps = 2 if (($Rfilez == 0) and ($Pfilez == 1));
    $wORps = 4 if (($Rfilez == 1) and ($Pfilez == 1));
}


## The file being searched (IPHOD.reals or IPHOD.pseudos)
sub load_input {
    my ($line, @line, $row, $cell);
    open (INPT, $input_file) || die "Cannot open ".$input_file."\n";
    print $input_file." opened.\n";
    $row = 0;
    while ($line = <INPT>) {
    chomp ($line);
    if ($line ne "") {
        $line =~ s/\"//g; # this removes any quotation marks
        $headerline = $line if ($row == 0);
        @line = split ("\t", $line);
        for ($cell=0;$cell<$queryrows+1;$cell++) {
        $data[$row][$cell] = @line[$cell];
        }
        $row += 1;
    }
    }
    $data_rows = $row;
    print "Data array Created, ".$data_rows." rows.\n";
    close(INPT);
    print $input_file." closed.\n\n";
}


sub search_data {
    my ($row, $cell, $c, $cc, $cel, $celle, $stopcons);
    print "Searching IPHOD for values within desired ranges.\n";

    ## SEARCH & PRINT OUT ROWS THAT MATCH THE CRITERION
    for ($row = 1; $row < $data_rows; $row++) {

        $match = 0; # When match = 0 after checking everything, the row is saved in output

        for ($cell = 0; $cell < $queryrows; $cell++) {
            if (($cell != 0) and (($cell < 3) || ($cell > 19))) {
                # Match is true whenever a val exceeds the min/max thresholds in search_vals.txt
                # Match only needs to be true once, then the word/pseudoword row is filtered out.
                if (($query[1][$cell+1] ne "") and ($match == 0) and ($query[1][$cell+1] > $data[$row][$cell])) {
                    $match += 1;
                }
                if (($query[2][$cell+1] ne "") and ($match == 0) and ($query[2][$cell+1] < $data[$row][$cell])) {
                    $match += 1;
                }
            }
        }


        # If all the values are within prescribed ranges, the save to file & up count
        if ($match == 0) {
            $number_of_matches += 1;
            for ($cc=0; $cc<$queryrows-2; $cc++) {
                print OUT $data[$row][$cc]."\t";
            }
            print OUT $data[$row][$queryrows-2]."\n";
        }
    }
}


## TERMINATE PROGRAM & NOTIFY
sub end_program { print "\n\n****** PROGRAM COMPLETE ******.\n\n"; }
